Module: Zeitwerk::Loader::Config

Extended by:
Internal
Includes:
RealModName
Included in:
Zeitwerk::Loader
Defined in:
lib/zeitwerk/loader/config.rb

Instance Attribute Summary collapse

Instance Method Summary collapse

Methods included from Internal

internal

Methods included from RealModName

#real_mod_name

Instance Attribute Details

#inflectorObject

: camelize(String, String) -> String



11
12
13
# File 'lib/zeitwerk/loader/config.rb', line 11

def inflector
  @inflector
end

#loggerObject

: call(String) -> void | debug(String) -> void | nil



14
15
16
# File 'lib/zeitwerk/loader/config.rb', line 14

def logger
  @logger
end

#nsfileObject

Basename of files that define namespaces. For example, if ‘nsfile` is ’ns.rb’, then ‘foo/ns.rb` defines the `Foo` namespace.

: String?



36
37
38
# File 'lib/zeitwerk/loader/config.rb', line 36

def nsfile
  @nsfile
end

#rootsObject (readonly)

Absolute paths of the root directories, mapped to their respective root namespaces:

'/Users/fxn/blog/app/channels' => Object,
'/Users/fxn/blog/app/adapters' => ActiveJob::QueueAdapters,
...

Stored in a hash to preserve order, easily handle duplicates, and have a fast lookup by directory.

This is a private collection maintained by the loader. The public interface for it is ‘push_dir` and `dirs`.

: Hash[String, Module]



29
30
31
# File 'lib/zeitwerk/loader/config.rb', line 29

def roots
  @roots
end

Instance Method Details

#collapse(*glob_patterns) ⇒ Object

Configure directories or glob patterns to be collapsed.

: (*(String | Pathname | Array[String | Pathname])) -> void



242
243
244
245
246
247
248
249
250
251
252
# File 'lib/zeitwerk/loader/config.rb', line 242

def collapse(*glob_patterns)
  glob_patterns = expand_paths(glob_patterns)
  mutex.synchronize do
    collapse_glob_patterns.merge(glob_patterns)
    new_collapse_dirs = expand_glob_patterns(glob_patterns)
    collapse_dirs.merge(new_collapse_dirs)
    new_collapse_dirs.each do |dir|
      collapse_parents << File.dirname(dir)
    end
  end
end

#dirs(namespaces: false, ignored: false) ⇒ Object

If ‘namespaces` is falsey (default), returns an array with the absolute paths of the root directories as strings. If truthy, returns a hash table instead. Keys are the absolute paths of the root directories as strings, values are their corresponding namespaces, class or module objects.

If ‘ignored` is falsey (default), ignored root directories are filtered out.

These are read-only collections, please add to them with ‘push_dir`.

: (?namespaces: boolish, ?ignored: boolish) -> Array | Hash[String, Module]



183
184
185
186
187
188
189
190
191
192
193
194
195
196
197
# File 'lib/zeitwerk/loader/config.rb', line 183

def dirs(namespaces: false, ignored: false)
  if namespaces
    if ignored || ignored_paths.empty?
      roots.clone
    else
      roots.reject { |root_dir, _namespace| ignored_path?(root_dir) }
    end
  else
    if ignored || ignored_paths.empty?
      roots.keys
    else
      roots.keys.reject { |root_dir| ignored_path?(root_dir) }
    end
  end.freeze
end

#do_not_eager_load(*paths) ⇒ Object

Let eager load ignore the given files or directories. The constants defined in those files are still autoloadable.

: (*(String | Pathname | Array[String | Pathname])) -> void



224
225
226
# File 'lib/zeitwerk/loader/config.rb', line 224

def do_not_eager_load(*paths)
  mutex.synchronize { eager_load_exclusions.merge(expand_paths(paths)) }
end

#enable_reloadingObject

You need to call this method before setup in order to be able to reload. There is no way to undo this, either you want to reload or you don’t.

: () -> void ! Zeitwerk::Error



203
204
205
206
207
208
209
210
211
212
213
# File 'lib/zeitwerk/loader/config.rb', line 203

def enable_reloading
  mutex.synchronize do
    break if @reloading_enabled

    if @setup
      raise Zeitwerk::Error, 'cannot enable reloading after setup'
    else
      @reloading_enabled = true
    end
  end
end

#ignore(*glob_patterns) ⇒ Object

Configure files, directories, or glob patterns to be totally ignored.

: (*(String | Pathname | Array[String | Pathname])) -> void



231
232
233
234
235
236
237
# File 'lib/zeitwerk/loader/config.rb', line 231

def ignore(*glob_patterns)
  glob_patterns = expand_paths(glob_patterns)
  mutex.synchronize do
    ignored_glob_patterns.merge(glob_patterns)
    ignored_paths.merge(expand_glob_patterns(glob_patterns))
  end
end

#initializeObject

: () -> void



100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
# File 'lib/zeitwerk/loader/config.rb', line 100

def initialize
  @inflector              = Zeitwerk::Inflector.new
  @logger                 = self.class.default_logger
  @tag                    = SecureRandom.hex(3)
  @initialized_at         = Time.now
  @roots                  = {}
  @nsfile                 = nil
  @ignored_glob_patterns  = Set.new
  @ignored_paths          = Set.new
  @collapse_glob_patterns = Set.new
  @collapse_dirs          = Set.new
  @collapse_parents       = Set.new
  @eager_load_exclusions  = Set.new
  @reloading_enabled      = false
  @on_setup_callbacks     = []
  @on_load_callbacks      = {}
  @on_unload_callbacks    = {}
end

#log!Object

Logs to ‘$stdout`, handy shortcut for debugging.

: () -> void



314
315
316
# File 'lib/zeitwerk/loader/config.rb', line 314

def log!
  @logger = ->(msg) { puts msg }
end

#on_load(cpath = :ANY, &block) ⇒ Object

Configure a block to be invoked once a certain constant path is loaded. Supports multiple callbacks, and if there are many, they are executed in the order in which they were defined.

loader.on_load('SomeApiClient') do |klass, _abspath|
  klass.endpoint = 'https://api.dev'
end

Can also be configured for any constant loaded:

loader.on_load do |cpath, value, abspath|
  # ...
end

: (String?) { (top, String) -> void } -> void ! TypeError

Raises:

  • (TypeError)


280
281
282
283
284
285
286
# File 'lib/zeitwerk/loader/config.rb', line 280

def on_load(cpath = :ANY, &block)
  raise TypeError, 'on_load only accepts strings' unless cpath.is_a?(String) || cpath == :ANY

  mutex.synchronize do
    (on_load_callbacks[cpath] ||= []) << block
  end
end

#on_setup(&block) ⇒ Object

Configure a block to be called after setup and on each reload. If setup was already done, the block runs immediately.

: () { () -> void } -> void



258
259
260
261
262
263
# File 'lib/zeitwerk/loader/config.rb', line 258

def on_setup(&block)
  mutex.synchronize do
    on_setup_callbacks << block
    block.call if @setup
  end
end

#on_unload(cpath = :ANY, &block) ⇒ Object

Configure a block to be invoked right before a certain constant is removed. Supports multiple callbacks, and if there are many, they are executed in the order in which they were defined.

loader.on_unload('Country') do |klass, _abspath|
  klass.clear_cache
end

Can also be configured for any removed constant:

loader.on_unload do |cpath, value, abspath|
  # ...
end

: (String?) { (top, String) -> void } -> void ! TypeError

Raises:

  • (TypeError)


303
304
305
306
307
308
309
# File 'lib/zeitwerk/loader/config.rb', line 303

def on_unload(cpath = :ANY, &block)
  raise TypeError, 'on_unload only accepts strings' unless cpath.is_a?(String) || cpath == :ANY

  mutex.synchronize do
    (on_unload_callbacks[cpath] ||= []) << block
  end
end

#push_dir(path, namespace: Object) ⇒ Object

Pushes ‘path` to the list of root directories.

Raises ‘Zeitwerk::Error` if `path` does not exist, or if another loader in the same process already manages that directory or one of its ascendants or descendants.

: (String | Pathname, namespace: Module) -> void ! Zeitwerk::Error



126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
# File 'lib/zeitwerk/loader/config.rb', line 126

def push_dir(path, namespace: Object)
  unless namespace.is_a?(Module) # Note that Class < Module.
    raise Zeitwerk::Error, "#{namespace.inspect} is not a class or module object, should be"
  end

  unless real_mod_name(namespace)
    raise Zeitwerk::Error, 'root namespaces cannot be anonymous'
  end

  abspath = File.expand_path(path)
  if @fs.dir?(abspath)
    raise_if_conflicting_root_dir(abspath)
    roots[abspath] = namespace
  else
    raise Zeitwerk::Error, "the root directory #{abspath} does not exist"
  end
end

#reloading_enabled?Boolean

: () -> bool

Returns:

  • (Boolean)


216
217
218
# File 'lib/zeitwerk/loader/config.rb', line 216

def reloading_enabled?
  @reloading_enabled
end

#tagObject

Returns the loader’s tag.

Implemented as a method instead of via attr_reader for symmetry with the writer below.

: () -> String



150
151
152
# File 'lib/zeitwerk/loader/config.rb', line 150

def tag
  @tag
end

#tag=(tag) ⇒ Object

Sets a tag for the loader, useful for logging.

: (to_s() -> String) -> void



157
158
159
# File 'lib/zeitwerk/loader/config.rb', line 157

def tag=(tag)
  @tag = tag.to_s
end