Module: Zeitwerk::Loader::Config

Extended by:
Internal
Includes:
RealModName
Included in:
Zeitwerk::Loader
Defined in:
lib/zeitwerk/loader/config.rb

Instance Attribute Summary collapse

Instance Method Summary collapse

Methods included from Internal

internal

Methods included from RealModName

#real_mod_name

Instance Attribute Details

#inflectorObject

Returns the value of attribute inflector.



11
12
13
# File 'lib/zeitwerk/loader/config.rb', line 11

def inflector
  @inflector
end

#loggerObject

Returns the value of attribute logger.



14
15
16
# File 'lib/zeitwerk/loader/config.rb', line 14

def logger
  @logger
end

#rootsObject (readonly)

Absolute paths of the root directories, mapped to their respective root namespaces:

"/Users/fxn/blog/app/channels" => Object,
"/Users/fxn/blog/app/adapters" => ActiveJob::QueueAdapters,
...

Stored in a hash to preserve order, easily handle duplicates, and have a fast lookup by directory.

This is a private collection maintained by the loader. The public interface for it is `push_dir` and `dirs`.



29
30
31
# File 'lib/zeitwerk/loader/config.rb', line 29

def roots
  @roots
end

Instance Method Details

#collapse(*glob_patterns) ⇒ Object

Configure directories or glob patterns to be collapsed.



207
208
209
210
211
212
213
# File 'lib/zeitwerk/loader/config.rb', line 207

def collapse(*glob_patterns)
  glob_patterns = expand_paths(glob_patterns)
  mutex.synchronize do
    collapse_glob_patterns.merge(glob_patterns)
    collapse_dirs.merge(expand_glob_patterns(glob_patterns))
  end
end

#dirs(namespaces: false) ⇒ Object

If `namespaces` is falsey (default), returns an array with the absolute paths of the root directories as strings. If truthy, returns a hash table instead. Keys are the absolute paths of the root directories as strings, values are their corresponding namespaces, class or module objects.

These are read-only collections, please add to them with `push_dir`.



155
156
157
158
159
160
161
# File 'lib/zeitwerk/loader/config.rb', line 155

def dirs(namespaces: false)
  if namespaces
    roots.clone
  else
    roots.keys
  end.freeze
end

#do_not_eager_load(*paths) ⇒ Object

Let eager load ignore the given files or directories. The constants defined in those files are still autoloadable.



189
190
191
# File 'lib/zeitwerk/loader/config.rb', line 189

def do_not_eager_load(*paths)
  mutex.synchronize { eager_load_exclusions.merge(expand_paths(paths)) }
end

#enable_reloadingObject

You need to call this method before setup in order to be able to reload. There is no way to undo this, either you want to reload or you don't.

Raises:



168
169
170
171
172
173
174
175
176
177
178
# File 'lib/zeitwerk/loader/config.rb', line 168

def enable_reloading
  mutex.synchronize do
    break if @reloading_enabled

    if @setup
      raise Zeitwerk::Error, "cannot enable reloading after setup"
    else
      @reloading_enabled = true
    end
  end
end

#ignore(*glob_patterns) ⇒ Object

Configure files, directories, or glob patterns to be totally ignored.



196
197
198
199
200
201
202
# File 'lib/zeitwerk/loader/config.rb', line 196

def ignore(*glob_patterns)
  glob_patterns = expand_paths(glob_patterns)
  mutex.synchronize do
    ignored_glob_patterns.merge(glob_patterns)
    ignored_paths.merge(expand_glob_patterns(glob_patterns))
  end
end

#initializeObject



86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
# File 'lib/zeitwerk/loader/config.rb', line 86

def initialize
  @inflector              = Zeitwerk::Inflector.new
  @logger                 = self.class.default_logger
  @tag                    = SecureRandom.hex(3)
  @initialized_at         = Time.now
  @roots                  = {}
  @ignored_glob_patterns  = Set.new
  @ignored_paths          = Set.new
  @collapse_glob_patterns = Set.new
  @collapse_dirs          = Set.new
  @eager_load_exclusions  = Set.new
  @reloading_enabled      = false
  @on_setup_callbacks     = []
  @on_load_callbacks      = {}
  @on_unload_callbacks    = {}
end

#log!Object

Logs to `$stdout`, handy shortcut for debugging.



279
280
281
# File 'lib/zeitwerk/loader/config.rb', line 279

def log!
  @logger = ->(msg) { puts msg }
end

#on_load(cpath = :ANY, &block) ⇒ Object

Configure a block to be invoked once a certain constant path is loaded. Supports multiple callbacks, and if there are many, they are executed in the order in which they were defined.

loader.on_load("SomeApiClient") do |klass, _abspath|
  klass.endpoint = "https://api.dev"
end

Can also be configured for any constant loaded:

loader.on_load do |cpath, value, abspath|
  # ...
end

Raises:

  • (TypeError)


243
244
245
246
247
248
249
# File 'lib/zeitwerk/loader/config.rb', line 243

def on_load(cpath = :ANY, &block)
  raise TypeError, "on_load only accepts strings" unless cpath.is_a?(String) || cpath == :ANY

  mutex.synchronize do
    (on_load_callbacks[cpath] ||= []) << block
  end
end

#on_setup(&block) ⇒ Object

Configure a block to be called after setup and on each reload. If setup was already done, the block runs immediately.



219
220
221
222
223
224
# File 'lib/zeitwerk/loader/config.rb', line 219

def on_setup(&block)
  mutex.synchronize do
    on_setup_callbacks << block
    block.call if @setup
  end
end

#on_unload(cpath = :ANY, &block) ⇒ Object

Configure a block to be invoked right before a certain constant is removed. Supports multiple callbacks, and if there are many, they are executed in the order in which they were defined.

loader.on_unload("Country") do |klass, _abspath|
  klass.clear_cache
end

Can also be configured for any removed constant:

loader.on_unload do |cpath, value, abspath|
  # ...
end

Raises:

  • (TypeError)


268
269
270
271
272
273
274
# File 'lib/zeitwerk/loader/config.rb', line 268

def on_unload(cpath = :ANY, &block)
  raise TypeError, "on_unload only accepts strings" unless cpath.is_a?(String) || cpath == :ANY

  mutex.synchronize do
    (on_unload_callbacks[cpath] ||= []) << block
  end
end

#push_dir(path, namespace: Object) ⇒ Object

Pushes `path` to the list of root directories.

Raises `Zeitwerk::Error` if `path` does not exist, or if another loader in the same process already manages that directory or one of its ascendants or descendants.

Raises:



111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
# File 'lib/zeitwerk/loader/config.rb', line 111

def push_dir(path, namespace: Object)
  # Note that Class < Module.
  unless namespace.is_a?(Module)
    raise Zeitwerk::Error, "#{namespace.inspect} is not a class or module object, should be"
  end

  unless real_mod_name(namespace)
    raise Zeitwerk::Error, "root namespaces cannot be anonymous"
  end

  abspath = File.expand_path(path)
  if dir?(abspath)
    raise_if_conflicting_directory(abspath)
    roots[abspath] = namespace
  else
    raise Zeitwerk::Error, "the root directory #{abspath} does not exist"
  end
end

#reloading_enabled?Boolean

Returns:

  • (Boolean)


181
182
183
# File 'lib/zeitwerk/loader/config.rb', line 181

def reloading_enabled?
  @reloading_enabled
end

#tagObject

Returns the loader's tag.

Implemented as a method instead of via attr_reader for symmetry with the writer below.



136
137
138
# File 'lib/zeitwerk/loader/config.rb', line 136

def tag
  @tag
end

#tag=(tag) ⇒ Object

Sets a tag for the loader, useful for logging.



143
144
145
# File 'lib/zeitwerk/loader/config.rb', line 143

def tag=(tag)
  @tag = tag.to_s
end