Module: Zeitwerk::Loader::Config

Extended by:
Internal
Included in:
Zeitwerk::Loader
Defined in:
lib/zeitwerk/loader/config.rb

Instance Attribute Summary collapse

Instance Method Summary collapse

Methods included from Internal

internal

Instance Attribute Details

#inflectorObject

Returns the value of attribute inflector.



10
11
12
# File 'lib/zeitwerk/loader/config.rb', line 10

def inflector
  @inflector
end

#loggerObject

Returns the value of attribute logger.



13
14
15
# File 'lib/zeitwerk/loader/config.rb', line 13

def logger
  @logger
end

#rootsObject (readonly)

Absolute paths of the root directories. Stored in a hash to preserve order, easily handle duplicates, have a fast lookup needed for detecting nested paths, and store namespaces as values.

"/Users/fxn/blog/app/channels" => Object,
"/Users/fxn/blog/app/adapters" => ActiveJob::QueueAdapters,
...

This is a private collection maintained by the loader. The public interface for it is `push_dir` and `dirs`.



27
28
29
# File 'lib/zeitwerk/loader/config.rb', line 27

def roots
  @roots
end

Instance Method Details

#collapse(*glob_patterns) ⇒ Object

Configure directories or glob patterns to be collapsed.



201
202
203
204
205
206
207
# File 'lib/zeitwerk/loader/config.rb', line 201

def collapse(*glob_patterns)
  glob_patterns = expand_paths(glob_patterns)
  mutex.synchronize do
    collapse_glob_patterns.merge(glob_patterns)
    collapse_dirs.merge(expand_glob_patterns(glob_patterns))
  end
end

#dirs(namespaces: false) ⇒ Object

If `namespaces` is falsey (default), returns an array with the absolute paths of the root directories as strings. If truthy, returns a hash table instead. Keys are the absolute paths of the root directories as strings, values are their corresponding namespaces, class or module objects.

These are read-only collections, please add to them with `push_dir`.



149
150
151
152
153
154
155
# File 'lib/zeitwerk/loader/config.rb', line 149

def dirs(namespaces: false)
  if namespaces
    roots.clone
  else
    roots.keys
  end.freeze
end

#do_not_eager_load(*paths) ⇒ Object

Let eager load ignore the given files or directories. The constants defined in those files are still autoloadable.



183
184
185
# File 'lib/zeitwerk/loader/config.rb', line 183

def do_not_eager_load(*paths)
  mutex.synchronize { eager_load_exclusions.merge(expand_paths(paths)) }
end

#enable_reloadingObject

You need to call this method before setup in order to be able to reload. There is no way to undo this, either you want to reload or you don't.

Raises:



162
163
164
165
166
167
168
169
170
171
172
# File 'lib/zeitwerk/loader/config.rb', line 162

def enable_reloading
  mutex.synchronize do
    break if @reloading_enabled

    if @setup
      raise Zeitwerk::Error, "cannot enable reloading after setup"
    else
      @reloading_enabled = true
    end
  end
end

#ignore(*glob_patterns) ⇒ Object

Configure files, directories, or glob patterns to be totally ignored.



190
191
192
193
194
195
196
# File 'lib/zeitwerk/loader/config.rb', line 190

def ignore(*glob_patterns)
  glob_patterns = expand_paths(glob_patterns)
  mutex.synchronize do
    ignored_glob_patterns.merge(glob_patterns)
    ignored_paths.merge(expand_glob_patterns(glob_patterns))
  end
end

#initializeObject



84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
# File 'lib/zeitwerk/loader/config.rb', line 84

def initialize
  @inflector              = Zeitwerk::Inflector.new
  @logger                 = self.class.default_logger
  @tag                    = SecureRandom.hex(3)
  @initialized_at         = Time.now
  @roots                  = {}
  @ignored_glob_patterns  = Set.new
  @ignored_paths          = Set.new
  @collapse_glob_patterns = Set.new
  @collapse_dirs          = Set.new
  @eager_load_exclusions  = Set.new
  @reloading_enabled      = false
  @on_setup_callbacks     = []
  @on_load_callbacks      = {}
  @on_unload_callbacks    = {}
end

#log!Object

Logs to `$stdout`, handy shortcut for debugging.



273
274
275
# File 'lib/zeitwerk/loader/config.rb', line 273

def log!
  @logger = ->(msg) { puts msg }
end

#on_load(cpath = :ANY, &block) ⇒ Object

Configure a block to be invoked once a certain constant path is loaded. Supports multiple callbacks, and if there are many, they are executed in the order in which they were defined.

loader.on_load("SomeApiClient") do |klass, _abspath|
  klass.endpoint = "https://api.dev"
end

Can also be configured for any constant loaded:

loader.on_load do |cpath, value, abspath|
  # ...
end

Raises:

  • (TypeError)


237
238
239
240
241
242
243
# File 'lib/zeitwerk/loader/config.rb', line 237

def on_load(cpath = :ANY, &block)
  raise TypeError, "on_load only accepts strings" unless cpath.is_a?(String) || cpath == :ANY

  mutex.synchronize do
    (on_load_callbacks[cpath] ||= []) << block
  end
end

#on_setup(&block) ⇒ Object

Configure a block to be called after setup and on each reload. If setup was already done, the block runs immediately.



213
214
215
216
217
218
# File 'lib/zeitwerk/loader/config.rb', line 213

def on_setup(&block)
  mutex.synchronize do
    on_setup_callbacks << block
    block.call if @setup
  end
end

#on_unload(cpath = :ANY, &block) ⇒ Object

Configure a block to be invoked right before a certain constant is removed. Supports multiple callbacks, and if there are many, they are executed in the order in which they were defined.

loader.on_unload("Country") do |klass, _abspath|
  klass.clear_cache
end

Can also be configured for any removed constant:

loader.on_unload do |cpath, value, abspath|
  # ...
end

Raises:

  • (TypeError)


262
263
264
265
266
267
268
# File 'lib/zeitwerk/loader/config.rb', line 262

def on_unload(cpath = :ANY, &block)
  raise TypeError, "on_unload only accepts strings" unless cpath.is_a?(String) || cpath == :ANY

  mutex.synchronize do
    (on_unload_callbacks[cpath] ||= []) << block
  end
end

#push_dir(path, namespace: Object) ⇒ Object

Pushes `path` to the list of root directories.

Raises `Zeitwerk::Error` if `path` does not exist, or if another loader in the same process already manages that directory or one of its ascendants or descendants.

Raises:



109
110
111
112
113
114
115
116
117
118
119
120
121
122
# File 'lib/zeitwerk/loader/config.rb', line 109

def push_dir(path, namespace: Object)
  # Note that Class < Module.
  unless namespace.is_a?(Module)
    raise Zeitwerk::Error, "#{namespace.inspect} is not a class or module object, should be"
  end

  abspath = File.expand_path(path)
  if dir?(abspath)
    raise_if_conflicting_directory(abspath)
    roots[abspath] = namespace
  else
    raise Zeitwerk::Error, "the root directory #{abspath} does not exist"
  end
end

#reloading_enabled?Boolean

Returns:

  • (Boolean)


175
176
177
# File 'lib/zeitwerk/loader/config.rb', line 175

def reloading_enabled?
  @reloading_enabled
end

#tagObject

Returns the loader's tag.

Implemented as a method instead of via attr_reader for symmetry with the writer below.



130
131
132
# File 'lib/zeitwerk/loader/config.rb', line 130

def tag
  @tag
end

#tag=(tag) ⇒ Object

Sets a tag for the loader, useful for logging.



137
138
139
# File 'lib/zeitwerk/loader/config.rb', line 137

def tag=(tag)
  @tag = tag.to_s
end